%
% Extracts an automaton from img and synthesizes it.
%
% If defFile is provided, writes the program specification to it.
% If resultFile is provided, writes the synthesized program to it.
%
% Returns the annotated automaton image.
%
function ann = run(img, defFile, resultFile)

[states,sm] = getStates(img);

[px, gb, nx, bb, bm] = getBoxes(img);

[t, l, tr, tm] = getTransitions(img, sm | bm, states);

if exist('defFile','var') && size(defFile,1) > 0 && ~exist('resultFile','var')
    exportDefinition(defFile, states, t, l, px, nx);
else
    if exist('defFile','var') && size(defFile,1) > 0
        results = synthesize(states, t, l, px, nx, defFile);
    else
        results = synthesize(states, t, l, px, nx);
    end
    if exist('resultFile','var')
        fd = fopen(resultFile,"w");
        fwrite(fd, results);
        fclose(fd);
    end
end

fh = figure();
imshow(img); hold on;
plotStates(states);
plotBoxes(px, gb, nx, bb);
plotTransitions(t, l, tr, states);
ann = saveAnnotatedImg(fh);
close(fh);
end

%
% Plots the example boxes for an automaton
%
function plotBoxes(px, good, nx, bad)
    for pIx=1:size(bad,1)
        rectangle('Position',bad(pIx).BoundingBox,'EdgeColor', [1 0 0], 'LineWidth', 2);
        text(bad(pIx).Centroid(1), bad(pIx).Centroid(2), nx(pIx), 'Color', [1 0 0]);
    end

    for pIx=1:size(good,1)
        rectangle('Position',good(pIx).BoundingBox,'EdgeColor', [0 1 0], 'LineWidth', 2);
        text(good(pIx).Centroid(1), good(pIx).Centroid(2), px(pIx), 'Color', [0 1 0]);
    end
end

%
% Plots the state boxes for an automaton
%
function plotStates(states)
    for stateIx=1:size(states,1)
        propss = states{stateIx,1};
        if (states{stateIx,2})
            color = [0 0 1];
        else
            color = [0 1 1];
        end
        rectangle('Position',propss.BoundingBox,'EdgeColor', color, 'LineWidth', 3);
    end
end

%
% Plots the transition arrows for an automaton
%
function plotTransitions(transitions, labels, regions, states)
    for ix=1:size(transitions,1)
        text(regions(ix).BoundingBox(1), regions(ix).BoundingBox(2), transitions(ix,1) + " --> " + transitions(ix,2) + " [" + labels(ix) + "]",'Color', [0 1 0]);
    end
end